@@ -124,9 +124,6 @@ static int fips_mode(void)
 	int fd;
 	char buf = 0;
 
-	if (access("/etc/system-fips", F_OK))
-		return 0;
-
 	fd = open("/proc/sys/crypto/fips_enabled", O_RDONLY);
 
 	if (fd < 0)
@@ -417,16 +414,8 @@ static int _setup(void)
 	if (_system(cmd, 1))
 		return 1;
 
-	if (!THE_LOOP_DEV)
-		THE_LOOP_DEV = crypt_loop_get_device();
-	if (!THE_LOOP_DEV) {
-		printf("Cannot find free loop device.\n");
-		return 1;
-	}
-	if (crypt_loop_device(THE_LOOP_DEV)) {
-		fd = crypt_loop_attach(THE_LOOP_DEV, test_loop_file, 0, 0, &ro);
-		close(fd);
-	}
+	fd = crypt_loop_attach(&THE_LOOP_DEV, test_loop_file, 0, 0, &ro);
+	close(fd);
 
 	tmp_file_1 = strdup(THE_LFILE_TEMPLATE);
 	if ((fd=mkstemp(tmp_file_1)) == -1) {
@@ -441,34 +430,15 @@ static int _setup(void)
 
 	_system("dmsetup create " DEVICE_EMPTY_name " --table \"0 10000 zero\"", 1);
 	_system("dmsetup create " DEVICE_ERROR_name " --table \"0 10000 error\"", 1);
-	if (!DEVICE_1)
-		DEVICE_1 = crypt_loop_get_device();
-	if (!DEVICE_1) {
-		printf("Cannot find free loop device.\n");
-		return 1;
-	}
-	if (crypt_loop_device(DEVICE_1)) {
-		_system(" [ ! -e " IMAGE1 " ] && bzip2 -dk " IMAGE1 ".bz2", 1);
-		fd = crypt_loop_attach(DEVICE_1, IMAGE1, 0, 0, &ro);
-		close(fd);
-	}
-	if (!DEVICE_2)
-		DEVICE_2 = crypt_loop_get_device();
-	if (!DEVICE_2) {
-		printf("Cannot find free loop device.\n");
-		return 1;
-	}
-	if (crypt_loop_device(DEVICE_2)) {
-		_system("dd if=/dev/zero of=" IMAGE_EMPTY " bs=1M count=4 2>/dev/null", 1);
-		fd = crypt_loop_attach(DEVICE_2, IMAGE_EMPTY, 0, 0, &ro);
-		close(fd);
-	}
-	if (!DEVICE_3)
-		DEVICE_3 = crypt_loop_get_device();
-	if (!DEVICE_3) {
-		printf("Cannot find free loop device.\n");
-		return 1;
-	}
+
+	_system(" [ ! -e " IMAGE1 " ] && bzip2 -dk " IMAGE1 ".bz2", 1);
+	fd = crypt_loop_attach(&DEVICE_1, IMAGE1, 0, 0, &ro);
+	close(fd);
+
+	_system("dd if=/dev/zero of=" IMAGE_EMPTY " bs=1M count=4 2>/dev/null", 1);
+	fd = crypt_loop_attach(&DEVICE_2, IMAGE_EMPTY, 0, 0, &ro);
+	close(fd);
+
 	/* Keymaterial offset is less than 8 sectors */
 	_system(" [ ! -e " EVL_HEADER_1 " ] && bzip2 -dk " EVL_HEADER_1 ".bz2", 1);
 	/* keymaterial offset aims into payload area */
@@ -1522,7 +1492,7 @@ static void LuksHeaderBackup(void)
 	crypt_free(cd);
 
 	// exercise luksOpen using backup header on block device
-	fd = crypt_loop_attach(DEVICE_3, BACKUP_FILE, 0, 0, &ro);
+	fd = crypt_loop_attach(&DEVICE_3, BACKUP_FILE, 0, 0, &ro);
 	close(fd);
 	OK_(fd < 0);
 	OK_(crypt_init(&cd, DEVICE_3));
@@ -1808,6 +1778,8 @@ static void VerityTest(void)
 	/* hash fail */
 	root_hash[1] = ~root_hash[1];
 	OK_(crypt_activate_by_volume_key(cd, CDEVICE_1, root_hash, 32, CRYPT_ACTIVATE_READONLY));
+	/* Be sure there was some read activity to mark device corrupted. */
+	_system("blkid " DMDIR CDEVICE_1, 0);
 	OK_(crypt_get_active_device(cd, CDEVICE_1, &cad));
 	EQ_(CRYPT_ACTIVATE_READONLY|CRYPT_ACTIVATE_CORRUPTED, cad.flags);
 	OK_(crypt_deactivate(cd, CDEVICE_1));
@@ -1816,6 +1788,7 @@ static void VerityTest(void)
 	/* data fail */
 	OK_(crypt_set_data_device(cd, DEVICE_1));
 	OK_(crypt_activate_by_volume_key(cd, CDEVICE_1, root_hash, 32, CRYPT_ACTIVATE_READONLY));
+	_system("blkid " DMDIR CDEVICE_1, 0);
 	OK_(crypt_get_active_device(cd, CDEVICE_1, &cad));
 	EQ_(CRYPT_ACTIVATE_READONLY|CRYPT_ACTIVATE_CORRUPTED, cad.flags);
 	OK_(crypt_deactivate(cd, CDEVICE_1));
@@ -1910,6 +1883,10 @@ static void TcryptTest(void)
 	OK_(crypt_deactivate(cd, CDEVICE_1));
 	crypt_free(cd);
 
+	// Following test uses non-FIPS algorithms in the cipher chain
+	if(_fips_mode)
+		return;
+
 	OK_(crypt_init(&cd, tcrypt_dev2));
 	params.keyfiles = NULL;
 	params.keyfiles_count = 0;
